// Copyright © 2014 
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions: The above copyright
// notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.

import UIKit

class WorkSpace: CanvasController {
    var animationTimer: Timer!
    var views = [View]()
    var circles = [Circle]()
    let duration = 3.0
    var container: View!
    var viewAnimationGroup: ViewAnimationGroup!
    var circleAnimationGroup: ViewAnimationGroup!

    override func setup() {
        canvas.backgroundColor = Color(red: 64, green: 177, blue: 239, alpha: 1.0)

        createViewsCircles()
        createAnimations()

        let containerAnim = ViewAnimation(duration: duration) {
            self.container.rotation += 2*M_PI
        }
        containerAnim.curve = .Linear
        containerAnim.repeats = true
        containerAnim.animate()

        animationTimer = Timer(interval: duration/4.0) {
            self.viewAnimationGroup.animate()
            self.circleAnimationGroup.animate()
        }
        animationTimer.start()
        animationTimer.fire()

    }

    func createViewsCircles() {
        container = View(frame: Rect(0,0,1,1))
        for _ in 1...4 {
            let v = View(frame: Rect(0,0,10,10))
            views.append(v)

            let c = Circle(center: v.center, radius: v.width/2.0)
            c.lineWidth = 10.0
            c.strokeColor = white
            c.fillColor = white
            circles.append(c)
            v.add(c)
            v.anchorPoint = Point(0.5,7.5)
            v.center = container.center
            container.add(v)
        }

        container.backgroundColor = C4Pink
        canvas.add(container)
        container.center = canvas.center
    }

    func createAnimations() {
        var canims = [ViewAnimation]()
        var vanims = [ViewAnimation]()
        for i in 0..<views.count {
            let v = views[i]
            let offset = Double(i) * 0.1 + 0.05
            let va = ViewAnimation(duration: duration/4.0 + 0.3) {
                v.rotation += M_PI
            }
            va.delay = offset
            vanims.append(va)

            let c = circles[i]
            let ca = ViewAnimation(duration: duration/8.0 + 0.15) {
                c.lineWidth = 0.0
            }
            ca.delay = offset
            ca.autoreverses = true
            ca.addCompletionObserver {
                ShapeLayer.disableActions = true
                c.lineWidth = 10.0
                ShapeLayer.disableActions = false
            }
            canims.append(ca)
        }
        viewAnimationGroup = ViewAnimationGroup(animations: vanims)
        circleAnimationGroup = ViewAnimationGroup(animations: canims)
    }
}